==> qubership-profiler-parsers-2.0.2-sources.jar: com/netcracker/profiler/sax/builders/TreeBuilderTrace.java <==
package com.netcracker.profiler.sax.builders;

import com.netcracker.profiler.chart.Provider;
import com.netcracker.profiler.io.Hotspot;
import com.netcracker.profiler.io.SuspendLog;
import com.netcracker.profiler.sax.raw.TreeTraceVisitor;
import com.netcracker.profiler.sax.values.ValueHolder;
import com.netcracker.profiler.util.ProfilerConstants;

import java.util.HashSet;

public class TreeBuilderTrace extends TreeTraceVisitor implements Provider<Hotspot> {
    private final Hotspot root;
    protected Hotspot[] callTree = new Hotspot[1000];
    protected Hotspot[] stack = new Hotspot[1000];

    public boolean started;
    private final SuspendLog suspendLog;
    private final SuspendLog.SuspendLogCursor suspendCursor;

    public TreeBuilderTrace(Hotspot root, SuspendLog suspendLog) {
        super(ProfilerConstants.PROFILER_V1);
        this.root = root;
        callTree[0] = root;
        stack[0] = new Hotspot(-1);
        this.suspendLog = suspendLog;
        this.suspendCursor = this.suspendLog.cursor();
    }

    protected void ensureStorage(int size) {
        if (size < callTree.length)
            return;
        Hotspot[] tmp = new Hotspot[callTree.length * 2];
        System.arraycopy(callTree, 0, tmp, 0, callTree.length);
        callTree = tmp;

        tmp = new Hotspot[stack.length * 2];
        System.arraycopy(stack, 0, tmp, 0, stack.length);
        stack = tmp;
    }

    @Override
    public void visitEnter(int methodId,
                           long lastAssemblyId,
                           long lastParentAssemblyId,
                           byte isReactorEndPoint,
                           byte isReactorFrame,
                           long reactorStartTime,
                           int  reactorDuration,
                           int blockingOperator,
                           int prevOperation,
                           int currentOperation,
                           int emit) {
        long time = getTime();
        int sp = getSp();

        Hotspot callTreeParent = callTree[sp];
        if (sp != 0)
            callTreeParent.suspensionTime += suspendCursor.moveTo(time);
        else {
            suspendCursor.skipTo(time);
            callTree[0].startTime = Math.min(callTree[0].startTime, time);
        }
        super.visitEnter(methodId, lastAssemblyId, lastParentAssemblyId,
                isReactorEndPoint, isReactorFrame,
                reactorStartTime, reactorDuration,
                blockingOperator, prevOperation, currentOperation,
                emit);
        sp++;
        ensureStorage(sp);
        Hotspot orCreateChild = callTreeParent.getOrCreateChild(methodId, lastParentAssemblyId);
        propagateReactorParams(lastAssemblyId, lastParentAssemblyId, isReactorEndPoint,
                isReactorFrame, blockingOperator, prevOperation, currentOperation,
                emit, orCreateChild);

        callTree[sp] = orCreateChild;
        Hotspot hs = stack[sp] = new Hotspot(methodId);
        hs.startTime = time;
        hs.endTime = time;
        hs.totalTime = (int) -time;
        if (reactorDuration != 0 && blockingOperator != 0) {
            orCreateChild.reactorDuration = reactorDuration;
            orCreateChild.reactorStartTime = reactorStartTime;
            hs.reactorDuration = reactorDuration;
            hs.reactorStartTime = reactorStartTime;
        }
    }

    private void propagateReactorParams(long lastAssemblyId,
                                        long lastParentAssemblyId,
                                        byte isReactorEndPoint,
                                        byte isReactorFrame,
                                        int blockingOperator,
                                        int prevOperation,
                                        int currentOperation,
                                        int emit,
                                        Hotspot orCreateChild) {
        orCreateChild.isReactorEndPoint = isReactorEndPoint;
        orCreateChild.isReactorFrame = isReactorFrame;
        if (lastAssemblyId != 0) {
            if (orCreateChild.lastAssemblyId == null) {
                orCreateChild.lastAssemblyId = new HashSet<>();
            }
            orCreateChild.lastAssemblyId.add(lastAssemblyId);
        }

        if (lastParentAssemblyId != 0) {
            orCreateChild.lastParentAssemblyId = lastParentAssemblyId;
        }

        if (blockingOperator != 0) {
            orCreateChild.blockingOperator = blockingOperator;
        }

        if (prevOperation != 0) {
            orCreateChild.prevOperation = prevOperation;
        }

        if (currentOperation != 0) {
            orCreateChild.currentOperation = currentOperation;
        }

        if (emit != 0) {
            orCreateChild.emit = emit;
        }
    }

    @Override
    public void visitEnter(int methodId) {
        visitEnter(methodId, 0, 0, (byte) 0, (byte) 0,
                0, 0, 0, 0,
                0, 0);
    }

    @Override
    public void visitExit() {
        long time = getTime();
        int sp = getSp();
        Hotspot hs = stack[sp];
        hs.suspensionTime += suspendCursor.moveTo(time);
        hs.totalTime += (int) time;
        hs.count++;
        callTree[sp].merge(hs);
        super.visitExit();
        sp--;
        if (sp != 0)
            return;
        callTree[0].endTime = Math.max(callTree[0].endTime, time);
        callTree[0].count++;
    }


    @Override
    public void visitLabel(int labelId, ValueHolder value, long assemblyId) {
        stack[getSp()].tag(0, labelId, 0, value, assemblyId);
    }

    @Override
    public void visitLabel(int labelId, ValueHolder value) {
        visitLabel(labelId, value, 0);
    }

    @Override
    public void visitEnd() {
        root.calculateTotalExecutions();
    }

    public Hotspot get() {
        return root;
    }
}

==> qubership-profiler-parsers-2.0.3-sources.jar: com/netcracker/profiler/sax/builders/TreeBuilderTrace.java <==
package com.netcracker.profiler.sax.builders;

import com.netcracker.profiler.chart.Provider;
import com.netcracker.profiler.io.Hotspot;
import com.netcracker.profiler.io.HotspotTag;
import com.netcracker.profiler.io.SuspendLog;
import com.netcracker.profiler.sax.raw.TreeTraceVisitor;
import com.netcracker.profiler.sax.values.ValueHolder;
import com.netcracker.profiler.util.ProfilerConstants;

import java.util.Arrays;

public class TreeBuilderTrace extends TreeTraceVisitor implements Provider<Hotspot> {
    private final Hotspot root;
    protected Hotspot[] callTree = new Hotspot[1000];
    protected Hotspot[] stack = new Hotspot[1000];
    protected HotspotTag.Builder[] tagStack = new HotspotTag.Builder[1000];

    private final SuspendLog suspendLog;
    private final SuspendLog.SuspendLogCursor suspendCursor;

    public TreeBuilderTrace(Hotspot root, SuspendLog suspendLog) {
        super(ProfilerConstants.PROFILER_V1);
        this.root = root;
        callTree[0] = root;
        stack[0] = new Hotspot(-1);
        this.suspendLog = suspendLog;
        this.suspendCursor = this.suspendLog.cursor();
    }

    protected void ensureStorage(int size) {
        if (size < callTree.length) {
            return;
        }
        callTree = Arrays.copyOf(callTree, callTree.length * 2);
        stack = Arrays.copyOf(stack, stack.length * 2);
        tagStack = Arrays.copyOf(tagStack, tagStack.length * 2);
    }

    @Override
    public void visitEnter(int methodId) {
        long time = getTime();
        int sp = getSp();

        Hotspot callTreeParent = callTree[sp];
        if (sp != 0)
            callTreeParent.suspensionTime += suspendCursor.moveTo(time);
        else {
            suspendCursor.skipTo(time);
            callTree[0].startTime = Math.min(callTree[0].startTime, time);
        }
        super.visitEnter(methodId);
        sp++;
        ensureStorage(sp);
        Hotspot orCreateChild = callTreeParent.getOrCreateChild(methodId);

        callTree[sp] = orCreateChild;
        Hotspot hs = stack[sp] = new Hotspot(methodId);
        hs.startTime = time;
        hs.endTime = time;
        hs.totalTime = (int) -time;
    }

    @Override
    public void visitExit() {
        long time = getTime();
        int sp = getSp();
        Hotspot hs = stack[sp];
        hs.suspensionTime += suspendCursor.moveTo(time);
        hs.totalTime += (int) time;
        hs.count++;
        HotspotTag.Builder tagBuilder = tagStack[sp];
        if (tagBuilder != null) {
            tagBuilder.forEachTag((tag) -> {
                tag.count = 1;
                tag.totalTime = hs.totalTime;
                hs.addTag(tag);
            });
            tagBuilder.clear();
        }
        callTree[sp].merge(hs);
        super.visitExit();
        sp--;
        if (sp != 0)
            return;
        callTree[0].endTime = Math.max(callTree[0].endTime, time);
        callTree[0].count++;
    }

    @Override
    public void visitLabel(int labelId, ValueHolder value) {
        HotspotTag.Builder tagBuilder = tagStack[getSp()];
        if (tagBuilder == null) {
            tagBuilder = new HotspotTag.Builder();
            tagStack[getSp()] = tagBuilder;
        }
        tagBuilder.addValue(labelId, value);
    }

    @Override
    public void visitEnd() {
        root.calculateTotalExecutions();
    }

    public Hotspot get() {
        return root;
    }
}
